@@ -16,8 +16,8 @@ from fido2.client import ClientError as Fido2ClientError
 from fido2.ctap1 import ApduError
 
 import solo
-from solo.cli.update import update
 import solo.fido2
+from solo.cli.update import update
 
 
 # https://pocoo-click.readthedocs.io/en/latest/commands/#nested-handling-and-contexts
@@ -107,6 +107,71 @@ def feedkernel(count, serial):
     print(f"Entropy after:  0x{open(entropy_info_file).read().strip()}")
 
 
+@click.command()
+@click.option("-s", "--serial", help="Serial number of Solo use")
+@click.option(
+    "--host", help="Relying party's host", default="solokeys.dev", show_default=True
+)
+@click.option("--user", help="User ID", default="they", show_default=True)
+@click.option(
+    "--prompt",
+    help="Prompt for user",
+    default="Touch your authenticator to generate a credential...",
+    show_default=True,
+)
+def make_credential(serial, host, user, prompt):
+    """Generate a credential.
+
+    Pass `--prompt ""` to output only the `credential_id` as hex.
+    """
+
+    import solo.hmac_secret
+
+    solo.hmac_secret.make_credential(
+        host=host, user_id=user, serial=serial, output=True, prompt=prompt
+    )
+
+
+@click.command()
+@click.option("-s", "--serial", help="Serial number of Solo use")
+@click.option("--host", help="Relying party's host", default="solokeys.dev")
+@click.option("--user", help="User ID", default="they")
+@click.option(
+    "--prompt",
+    help="Prompt for user",
+    default="Touch your authenticator to generate a reponse...",
+    show_default=True,
+)
+@click.argument("credential-id")
+@click.argument("challenge")
+def challenge_response(serial, host, user, prompt, credential_id, challenge):
+    """Uses `hmac-secret` to implement a challenge-response mechanism.
+
+    We abuse hmac-secret, which gives us `HMAC(K, hash(challenge))`, where `K`
+    is a secret tied to the `credential_id`. We hash the challenge first, since
+    a 32 byte value is expected (in original usage, it's a salt).
+
+    This means that we first need to setup a credential_id; this depends on the
+    specific authenticator used. To do this, use `solo key make-credential`.
+
+    If so desired, user and relying party can be changed from the defaults.
+
+    The prompt can be suppressed using `--prompt ""`.
+    """
+
+    import solo.hmac_secret
+
+    solo.hmac_secret.simple_secret(
+        credential_id,
+        challenge,
+        host=host,
+        user_id=user,
+        serial=serial,
+        prompt=prompt,
+        output=True,
+    )
+
+
 @click.command()
 @click.option("-s", "--serial", help="Serial number of Solo use")
 @click.option(
@@ -203,11 +268,12 @@ def reset(serial):
 
 
 @click.command()
+@click.option("--pin", help="PIN for to access key")
 @click.option("-s", "--serial", help="Serial number of Solo to use")
 @click.option(
     "--udp", is_flag=True, default=False, help="Communicate over UDP with software key"
 )
-def verify(serial, udp):
+def verify(pin, serial, udp):
     """Verify key is valid Solo Secure or Solo Hacker."""
 
     if udp:
@@ -216,11 +282,44 @@ def verify(serial, udp):
     # Any longer and this needs to go in a submodule
     print("Please press the button on your Solo key")
     try:
-        cert = solo.client.find(serial).make_credential()
-    except Fido2ClientError:
-        print("Error getting credential, is your key in bootloader mode?")
-        print("Try: `solo program aux leave-bootloader`")
-        sys.exit(1)
+        cert = solo.client.find(serial).make_credential(pin=pin)
+    except ValueError as e:
+        # python-fido2 library pre-emptively returns `ValueError('PIN required!')`
+        # instead of trying, and returning  `CTAP error: 0x36 - PIN_REQUIRED`
+        if "PIN required" in str(e):
+            print("Your key has a PIN set. Please pass it using `--pin <your PIN>`")
+            sys.exit(1)
+        raise
+
+    except Fido2ClientError as e:
+        cause = str(e.cause)
+        # error 0x31
+        if "PIN_INVALID" in cause:
+            print("Your key has a different PIN. Please try to remember it :)")
+            sys.exit(1)
+        # error 0x34 (power cycle helps)
+        if "PIN_AUTH_BLOCKED" in cause:
+            print(
+                "Your key's PIN authentication is blocked due to too many incorrect attempts."
+            )
+            print("Please plug it out and in again, then again!")
+            print(
+                "Please be careful, after too many incorrect attempts, the key will fully block."
+            )
+            sys.exit(1)
+        # error 0x32 (only reset helps)
+        if "PIN_BLOCKED" in cause:
+            print(
+                "Your key's PIN is blocked. To use it again, you need to fully reset it."
+            )
+            print("You can do this using: `solo key reset`")
+            sys.exit(1)
+        # error 0x01
+        if "INVALID_COMMAND" in cause:
+            print("Error getting credential, is your key in bootloader mode?")
+            print("Try: `solo program aux leave-bootloader`")
+            sys.exit(1)
+        raise
 
     solo_fingerprint = b"r\xd5\x831&\xac\xfc\xe9\xa8\xe8&`\x18\xe6AI4\xc8\xbeJ\xb8h_\x91\xb0\x99!\x13\xbb\xd42\x95"
     hacker_fingerprint = b"\xd0ml\xcb\xda}\xe5j\x16'\xc2\xa7\x89\x9c5\xa2\xa3\x16\xc8Q\xb3j\xd8\xed~\xd7\x84y\xbbx~\xf7"
@@ -275,6 +374,8 @@ key.add_command(rng)
 rng.add_command(hexbytes)
 rng.add_command(raw)
 rng.add_command(feedkernel)
+key.add_command(make_credential)
+key.add_command(challenge_response)
 key.add_command(reset)
 key.add_command(update)
 key.add_command(probe)
