@@ -21,13 +21,13 @@ from fido2.client import Fido2Client
 from fido2.ctap import CtapError
 from fido2.ctap1 import CTAP1
 from fido2.ctap2 import CTAP2
-from fido2.hid import CtapHidDevice, CTAPHID
+from fido2.hid import CTAPHID, CtapHidDevice
 from fido2.utils import Timeout
 from intelhex import IntelHex
 
-from solo.commands import SoloBootloader, SoloExtension
 import solo.exceptions
 from solo import helpers
+from solo.commands import SoloBootloader, SoloExtension
 
 
 def find(solo_serial=None, retries=5, raw_device=None):
@@ -68,6 +68,7 @@ class SoloClient:
     def __init__(self,):
         self.origin = "https://example.org"
         self.host = "example.org"
+        self.user_id = b"they"
         self.exchange = self.exchange_hid
         self.do_reboot = True
 
@@ -208,11 +209,13 @@ class SoloClient:
     def reset(self,):
         self.ctap2.reset()
 
-    def make_credential(self,):
+    def make_credential(self, pin=None):
         rp = {"id": self.host, "name": "example site"}
-        user = {"id": b"abcdef", "name": "example user"}
+        user = {"id": self.user_id, "name": "example user"}
         challenge = "Y2hhbGxlbmdl"
-        attest, data = self.client.make_credential(rp, user, challenge, exclude_list=[])
+        attest, data = self.client.make_credential(
+            rp, user, challenge, exclude_list=[], pin=pin
+        )
         try:
             attest.verify(data.hash)
         except AttributeError:
